#ifndef RP_PROCESS_LOG_FRAME_H
#define RP_PROCESS_LOG_FRAME_H

#include <memory>
#include <chrono>
#include <map>
#include <string>
#include <mutex>
#include <vector>
#include <functional>
#include <sstream>
#ifdef __linux
#include <jsoncpp/json/json.h>
#endif

#ifdef _WIN32
#include <json/json.h>
#endif
#include <rp_process_log/process_log_define.h>
#include <rp_process_log/process_log_cpp_base.h>

#ifdef __linux
#define INTERFACE_HIDED
//__attribute__( ( visibility( "hidden" ) ) )
#endif

#ifdef _WIN32
#define INTERFACE_HIDED
#endif

namespace rp_process_log
{

CLASS_FORWARD( ProcessLogFrame )

extern std::map<std::string, std::function<ProcessLogCppBasePtr( std::string module_name )>> g_module_generators;

/**
 * @struct LogHelper
 * @brief  日志信息暂存结构体
 */
struct LogHelper
{
  /*日志关键字，日志处理依据*/
  std::string key;
  /*日志等级*/
  LogLevel level;
  /*日志内容*/
  std::string text;
  /*当前文件*/
  std::string filename;
  /*当前行*/
  uint32_t line;
  /*当前函数*/
  std::string funcname;
};

#define LOG_INSTANCE rp_process_log::ProcessLogFrame::getInstance()

#define RP_PROC_LOG_STREAM( x, k, lv, lg )                                                         \
  do                                                                                               \
  {                                                                                                \
    if ( lg->getLogLevel() >= lv )                                                                 \
    {                                                                                              \
      std::stringstream __sstream;                                                                 \
      __sstream << x;                                                                              \
      lg->log( { k, lv, std::move( __sstream.str() ), __FILE__, __LINE__, __PRETTY_FUNCTION__ } ); \
    }                                                                                              \
  } while ( false )

#define RP_DEBUG_STREAM_K( x, k ) RP_PROC_LOG_STREAM( x, k, rp_process_log::LV_DEBUG, LOG_INSTANCE )
#define RP_INFO_STREAM_K( x, k )  RP_PROC_LOG_STREAM( x, k, rp_process_log::LV_INFO, LOG_INSTANCE )
#define RP_WARN_STREAM_K( x, k )  RP_PROC_LOG_STREAM( x, k, rp_process_log::LV_WARN, LOG_INSTANCE )
#define RP_ERROR_STREAM_K( x, k ) RP_PROC_LOG_STREAM( x, k, rp_process_log::LV_ERROR, LOG_INSTANCE )
#define RP_FATAL_STREAM_K( x, k ) RP_PROC_LOG_STREAM( x, k, rp_process_log::LV_FATAL, LOG_INSTANCE )

#define RP_DEBUG_STREAM( x ) RP_DEBUG_STREAM_K( x, "" )
#define RP_INFO_STREAM( x )  RP_INFO_STREAM_K( x, "" )
#define RP_WARN_STREAM( x )  RP_WARN_STREAM_K( x, "" )
#define RP_ERROR_STREAM( x ) RP_ERROR_STREAM_K( x, "" )
#define RP_FATAL_STREAM( x ) RP_FATAL_STREAM_K( x, "" )

#define RP_PROC_LOG( lg, k, lv, fmt, ... )                                                                         \
  do                                                                                                               \
  {                                                                                                                \
    if ( lg->getLogLevel() >= lv ) lg->logf( k, lv, __FILE__, __LINE__, __PRETTY_FUNCTION__, fmt, ##__VA_ARGS__ ); \
  } while ( false )

#define RP_DEBUG_K( k, fmt, ... ) RP_PROC_LOG( LOG_INSTANCE, k, rp_process_log::LV_DEBUG, fmt, ##__VA_ARGS__ )
#define RP_INFO_K( k, fmt, ... )  RP_PROC_LOG( LOG_INSTANCE, k, rp_process_log::LV_INFO, fmt, ##__VA_ARGS__ )
#define RP_WARN_K( k, fmt, ... )  RP_PROC_LOG( LOG_INSTANCE, k, rp_process_log::LV_WARN, fmt, ##__VA_ARGS__ )
#define RP_ERROR_K( k, fmt, ... ) RP_PROC_LOG( LOG_INSTANCE, k, rp_process_log::LV_ERROR, fmt, ##__VA_ARGS__ )
#define RP_FATAL_K( k, fmt, ... ) RP_PROC_LOG( LOG_INSTANCE, k, rp_process_log::LV_FATAL, fmt, ##__VA_ARGS__ )

#define RP_DEBUG( fmt, ... ) RP_DEBUG_K( "", fmt, ##__VA_ARGS__ )
#define RP_INFO( fmt, ... )  RP_INFO_K( "", fmt, ##__VA_ARGS__ )
#define RP_WARN( fmt, ... )  RP_WARN_K( "", fmt, ##__VA_ARGS__ )
#define RP_ERROR( fmt, ... ) RP_ERROR_K( "", fmt, ##__VA_ARGS__ )
#define RP_FATAL( fmt, ... ) RP_FATAL_K( "", fmt, ##__VA_ARGS__ )

/**
 * @class ProcessLogFrame
 * @brief process log框架
 * @author luyp
 */

class ProcessLogFrame
{
 public:
  ProcessLogFrame();
  virtual ~ProcessLogFrame();

  /**
   * @brief 日志系统初始化
   * @param [in] json_cfg      初始化json字符串
   * @param [in] module_name   模块名称
   * @return 是否调用成功
   */
  bool init( const char *json_cfg, const char *module_name );
  /**
   * @brief 记录日志
   * @param [in] key    日志关键字(用于分类)
   * @param [in] lv     日志等级
   * @param [in] text   日志内容
   * @param [in] filename  当前cpp文件名
   * @param [in] line      当前文件行号
   * @return 是否调用成功
   */
  bool log( std::string key, LogLevel lv, std::string &text, const char *filename, uint32_t line,
            const char *funcname );
  /**
   * @brief 记录日志
   * @param [in] key  日志关键字(用于分类)
   * @param [in] lv   日志等级
   * @param [in] text   日志内容(右值引用优化)
   * @param [in] filename  当前cpp文件名
   * @param [in] line      当前文件行号
   * @return 是否调用成功
   */
  bool log( std::string key, LogLevel lv, std::string &&text, const char *filename, uint32_t line,
            const char *funcname );
  /**
   * @brief 记录日志
   * @param [in] log  日志信息暂存结构体
   * @return 是否调用成功
   */
  bool log( LogHelper &log );
  /**
   * @brief 记录日志
   * @param [in] log  日志信息暂存结构体
   * @return 是否调用成功
   */
  bool log( LogHelper &&log );

  /**
   *  @brief 不定参数日志接口
   *  @param [in] key  日志关键字(用于分类)
   *  @param [in] lv   日志等级
   *  @param [in] filename  当前cpp文件名
   *  @param [in] line      当前文件行号
   *  @param [in] fmt       日志内容格式化
   *  @param [in] ...       不定参数
   *  @return 是否调用成功
   */

  bool logf( std::string key, LogLevel lv, const char *filename, uint32_t line, const char *funcname, const char *fmt,
             ... );

  /**
   *  @brief 日志单例接口
   *  @return 返回日志单例对象
   */

  INTERFACE_HIDED static ProcessLogFramePtr getInstance()
  {
    static ProcessLogFramePtr instance;
    if ( !instance )
    {
      instance.reset( new ProcessLogFrame() );
    }

    return instance;
  }

  /**
   *  @brief  设置全局日志等级
   *  @param [in] lv 日志等级
   */
  void setLogLevel( LogLevel lv ) { global_level_ = lv; }
  /**
   *  @brief  返回全局日志等级
   *  @return 日志等级
   */
  LogLevel getLogLevel() { return global_level_; }

 protected:
  /**
   *  @brief  检查是否初始化，如果未初始化则初始化
   *  @return 返回检查是否成功
   */

  bool checkInitialized();
  bool inited_           = false;
  LogLevel global_level_ = rp_process_log::LogLevel::LV_DEBUG;
  std::map<std::string, ProcessLogCppBasePtr> modules_map_;
  std::vector<ProcessLogCppBasePtr> modules_;
  std::string process_module_name_;
  INTERFACE_HIDED static ProcessLogFramePtr instance_;
};

}  // namespace rp_process_log
#endif  // RP_PROCESS_LOG_FRAME_H
