#include <stdio.h>
#include <string.h>
#include <string>
#include <chrono>
#include <iostream>
#include <vector>
#include <map>
#include <rp_process_log/process_log_define.h>
#include <rp_process_log/process_log_cpp_formater.h>

namespace rp_process_log
{

// REGIST_RP_PROCESS_LOG_MODULE("formater", ProcessLogCppFormater)

constexpr static uint32_t log_show_time   = 0x01;
constexpr static uint32_t log_show_level  = 0x02;
constexpr static uint32_t log_show_file   = 0x04;
constexpr static uint32_t log_show_line   = 0x08;
constexpr static uint32_t log_show_key    = 0x10;
constexpr static uint32_t log_show_module = 0x20;
constexpr static uint32_t log_show_func   = 0x40;
constexpr static uint32_t log_show_color  = 0x80;

constexpr static uint32_t time_buff_len = 50;

static std::vector<std::string> g_logLevelMap = { "[FATAL] ", "[ERROR] ", "[WARN]  ", "[INFO]  ", "[DEBUG] " };

static std::map<std::string, uint32_t> g_elementMap = {
    { CONFIG_VAL_TIME, log_show_time },     { CONFIG_VAL_LEVEL, log_show_level },
    { CONFIG_VAL_FILE, log_show_file },     { CONFIG_VAL_LINE, log_show_line },
    { CONFIG_VAL_KEY, log_show_key },       { CONFIG_VAL_MODULE, log_show_module },
    { CONFIG_VAL_FUNCTION, log_show_func }, { CONFIG_VAL_COLOR, log_show_color } };

static std::vector<std::pair<std::string, std::string>> g_colored_chars = { { "\033[31m", "\033[0m" },
                                                                            { "\033[31m", "\033[0m" },
                                                                            { "\033[33m", "\033[0m" },
                                                                            { "\033[49m", "\033[0m" },
                                                                            { "\033[32m", "\033[0m" } };

ProcessLogCppFormatter::ProcessLogCppFormatter( const char *module )
    : ProcessLogCppBase( module ), show_element_( 0x80 )
{
}

bool ProcessLogCppFormatter::init( const char * ) { return true; }

bool ProcessLogCppFormatter::process( LogRecordInfoPtr info )
{
  if ( !info )
  {
    return false;
  }

  auto level = info->level;
  if ( level <= LV_NULL )
  {
    return false;
  }
  std::string srclog = info->log;

  std::string log;
  log.reserve( time_buff_len + srclog.size() + 16 );

  auto ttime = std::chrono::system_clock::to_time_t( info->time );

  if ( show_element_ & log_show_level ) log += g_logLevelMap[ level - 1 ];

  if ( show_element_ & log_show_time )
  {
    char nowt[ time_buff_len ] = { 0 };
    strftime( nowt, sizeof( nowt ), "%Y-%m-%d %H:%M:%S", localtime( &ttime ) );

    std::string now_str = nowt;
    auto ms             = std::chrono::duration_cast<std::chrono::microseconds>( info->time.time_since_epoch() );
    now_str += ".";

    auto time_str = std::to_string( ms.count() % 1000000 );
    if ( time_str.size() < 6 )
    {
      time_str = std::string( 6 - time_str.size(), '0' ) + time_str;
    }
    now_str += time_str;

    log += "[";
    log += now_str;  // ctime(&ttime);
    log += "] ";
  }

  if ( show_element_ & log_show_module )
  {
    log += "[";
    log += module_name_;  // ctime(&ttime);
    if ( show_element_ & log_show_line )
    {
      log += ":";
      log += std::to_string( info->line );
    }
    log += "] ";
  }

  if ( show_element_ & log_show_key )
  {
    log += "[";
    log += info->key;
    log += "] ";
  }

  if ( show_element_ & log_show_file )
  {
    log += "[";
    log += info->filename;
    if ( show_element_ & log_show_line )
    {
      log += ":";
      log += std::to_string( info->line );
    }
    log += "] ";
  }

  if ( show_element_ & log_show_func )
  {
    log += "[";
    log += info->funcname;
    log += "] ";
  }

  if ( log_max_size_ )
  {
    if ( srclog.size() > log_max_size_ )
    {
      srclog.resize( log_max_size_ );
      srclog += "...[truncated]";
    }
  }
  log += srclog;
  log += "\n";

  if ( show_element_ & log_show_color )
  {
    auto &color_chars = g_colored_chars[ level - 1 ];
    log               = color_chars.first + log + color_chars.second;
  }

  info->log.swap( log );

#ifdef DEBUG
  DEBUG_LOG( log );
#endif

  return ProcessLogCppBase::process( info );
}
#define GET_CONFIG_INFO_DATA( v, d ) g_elementMap.count( v ) ? g_elementMap[ v ] : d

void ProcessLogCppFormatter::setConfig( const char *key, const char *val )
{
  if ( strcmp( key, CONFIG_KEY_SHOW_INFO ) == 0 )
  {
    show_element_ |= GET_CONFIG_INFO_DATA( val, LV_NULL );
  }

  if ( strcmp( key, CONFIG_KEY_LOG_MAX_SIZE ) == 0 )
  {
    log_max_size_ = std::atoi( val );
  }
}

}  // namespace rp_process_log
